/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten;

import java.util.Arrays;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.OperandenListe;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Testet die Ausführung der Addition innerhalb des Arithmetikhandlers.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ArithmetikHandlerAddition {

	/**
	 * der verwendete Handler.
	 */
	private static final ArithmetikHandler HANDLER = new ArithmetikHandler();

	/**
	 * der Additionsoperator.
	 */
	private static final Operator OPERATOR = ArithmetikOperators.ADDITION;

	/**
	 * Ausführung der Operation ohne Operanden.
	 */
	@Test
	public void keineOperanden() {
		try {
			ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get());
			Assert.fail("Es wurde nicht erkannt, das keine Argumente angegeben wurden.");
		} catch (final ArgumentFehler fehler) {
			if (!fehler.getSubTyp().equals(UdaFehlerSubtyp.ANZAHL)) {
				Assert.fail("Der Fehler-Untertyp ist nicht: Falsche Anzahl von Parametern, sondern: "
						+ fehler.getSubTyp());
			}
		}
	}

	/**
	 * Ausführung der Operation mit zu wenigen Operanden.
	 */
	@Test(expected = ArgumentFehler.class)
	public void zuWenigOperanden() {
		ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12));
	}

	/**
	 * Ausführung der Addition.
	 */
	@Test
	public void testAddition() {
		Object result = ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR,
				OperandenListe.get(12, 13));
		Assert.assertEquals("Summe ist nicht korrekt", 25L, ((Number) result)
				.longValue());
		Assert.assertEquals("Addition liefert keine Ganzzahl", Long.class, result
				.getClass());

		result = ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR,
				OperandenListe.get(12, 13.0));
		Assert.assertEquals("Summe ist nicht korrekt", 25.0, ((Number) result)
				.doubleValue(), 0);
		Assert.assertEquals("Addition liefert keine Fließkommazahl", Double.class,
				result.getClass());

		result = ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR,
				OperandenListe.get(12.0, 13));
		Assert.assertEquals("Summe ist nicht korrekt", 25.0, ((Number) result)
				.doubleValue());
		Assert.assertEquals("Addition liefert keine Fließkommazahl", Double.class,
				result.getClass());

		result = ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR,
				OperandenListe.get(12.0, 13.0));
		Assert.assertEquals("Summe ist nicht korrekt", 25.0, ((Number) result)
				.doubleValue());
		Assert.assertEquals("Addition liefert keine Fließkommazahl", Double.class,
				result.getClass());

	}

	/**
	 * Ausführung der Operation mit zu vielen Argumenten.
	 */
	@Test(expected = ArgumentFehler.class)
	public void zuvieleOperanden() {
		ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12, 13, 14));
	}

	/**
	 * Ausführung der Operation mit Operanden vom falschen Typ.
	 */
	@Test(expected = ArgumentFehler.class)
	public void falscheOperanden() {
		ArithmetikHandlerAddition.HANDLER.perform(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get("Ein String"));
	}

	/**
	 * ermittelt, ob die erwarteten Operatoren vom Handler unterstützt werden.
	 */
	@Test
	public void testGetHandledOperators() {
		Assert.assertFalse("Anzahl der Operatoren ist 0", 0 == ArithmetikHandlerAddition.HANDLER
				.getHandledOperators().length);
		Assert.assertTrue(Arrays.asList(ArithmetikHandlerAddition.HANDLER.getHandledOperators())
				.contains(ArithmetikHandlerAddition.OPERATOR));
	}

	/**
	 * testet die Funktion validiereHandler für verschiedene
	 * Operandenkombinationen.
	 */
	@Test
	public void testValidiereHandlerOperatorListOfQextendsObject() {
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get()).isValid());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get()).isRichtigeAnzahl());
		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get()).isRichtigerTyp());

		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12)).isValid());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get()).isRichtigeAnzahl());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get()).isRichtigeAnzahl());

		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12.0)).isValid());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get()).isRichtigeAnzahl());
		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get()).isRichtigerTyp());

		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get("Ein String")).isValid());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get("Ein String"))
				.isRichtigeAnzahl());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get("Ein String"))
				.isRichtigerTyp());

		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(1, "Ein String")).isValid());
		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(1, "Ein String"))
				.isRichtigeAnzahl());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(1, "Ein String"))
				.isRichtigerTyp());

		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12, 13, 14)).isValid());
		Assert.assertFalse(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12, 13, 14))
				.isRichtigeAnzahl());
		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12, 13, 14)).isRichtigerTyp());

		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12.0, 13.0)).isValid());
		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12.0, 13)).isValid());
		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12, 13.0)).isValid());
		Assert.assertTrue(ArithmetikHandlerAddition.HANDLER
				.validiereHandler(ArithmetikHandlerAddition.OPERATOR, OperandenListe.get(12, 13)).isValid());
	}
}
