/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten;

import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.ibv.uda.interpreter.anweisung.UdaAktivitaet;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitDauer;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * der Handler für Threadspezifische Operationen.<br>
 * Folgende Operationen werden unterstützt:
 * <ul>
 * <li>beenden: zum Beenden eines Threads (einer {@link UdaAktivitaet})</li>
 * <li>pause: zum Anhalten eines Thread (einer {@link UdaAktivitaet}) für eine
 * angegebene Zeit</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ThreadHandler extends AbstractUdaHandler {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return ThreadOperatoren.OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {

		erzeugeHandlerFehler(operator, operanden);

		Object result = null;

		if (ThreadOperatoren.BEENDEN.equals(operator)) {
			final UdaAktivitaet thread = (UdaAktivitaet) operanden.get(0);
			result = LogischerWert.valueOf(thread.isRunning());
			thread.beenden();
		} else {
			if (ThreadOperatoren.PAUSE.equals(operator)) {
				final ZeitDauer dauer = (ZeitDauer) operanden.get(0);
				try {
					Thread.sleep(dauer.getMilliSekunden());
					result = LogischerWert.valueOf(true);
				} catch (final InterruptedException e) {
					LOGGER.error(e.getLocalizedMessage(), e);
					result = LogischerWert.valueOf(false);
				}
			}
		}

		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		boolean anzahlOk = false;
		boolean typOk = false;
		if (ThreadOperatoren.BEENDEN.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				typOk = true;
			}
		} else if (ThreadOperatoren.PAUSE.equals(operator)) {
			if (operanden.size() == 1) {
				anzahlOk = true;
				if (operanden.get(0) instanceof ZeitDauer) {
					typOk = true;
				}
			}
		}
		return new HandlerValidation(anzahlOk, typOk);
	}
}
