/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObjectInfo;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.KonfigurationsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * abstrakte Klasse zur Kapselung von Datenverteiler-Objekten.<br>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class DavDaten implements Struktur {

	/**
	 * die Funktion erzeugt ein kapselndes Objekt entsprechend dem Typ des
	 * übergebenen Datenverteilerobjekts.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param objekt
	 *            das Datenverteilerobjekt.
	 * @return das Uda-Objekt
	 */
	public static DavDaten erzeugeObjekt(final ClientDavInterface verbindung, final Object objekt) {
		DavDaten result = null;
		if (objekt instanceof IntegerValueState) {
			result = new IntegerValueStateObjekt(verbindung, (IntegerValueState) objekt);
		} else if (objekt instanceof IntegerValueRange) {
			result = new IntegerValueRangeObjekt(verbindung, (IntegerValueRange) objekt);
		} else if (objekt instanceof SystemObjectInfo) {
			result = new SystemObjektInfoObjekt(verbindung, (SystemObjectInfo) objekt);
		} else if (objekt instanceof SystemObjectType) {
			result = new TypObjekt(verbindung, (SystemObjectType) objekt);
		} else if (objekt instanceof ObjectSet) {
			result = new MengenObjekt(verbindung, (ObjectSet) objekt);
		} else if (objekt instanceof AttributeGroup) {
			result = new AtgObjekt(verbindung, (AttributeGroup) objekt);
		} else if (objekt instanceof Aspect) {
			result = new AspektObjekt(verbindung, (Aspect) objekt);
		} else if (objekt instanceof Attribute) {
			result = new AttributObjekt(verbindung, (Attribute) objekt);
		} else if (objekt instanceof AttributeListDefinition) {
			result = new AttributListDefinitionObjekt(verbindung, (AttributeListDefinition) objekt);
		} else if (objekt instanceof AttributeType) {
			result = new AttributeTypeObjekt(verbindung, (AttributeType) objekt);
		} else if (objekt instanceof ConfigurationObject) {
			result = new KonfigurationsObjekt(verbindung, (ConfigurationObject) objekt);
		} else if (objekt instanceof DynamicObject) {
			result = new DynamischesObjekt(verbindung, (DynamicObject) objekt);
		} else if (objekt instanceof ResultData) {
			result = new DavDatenSatz(verbindung, (ResultData) objekt);
		} else if (objekt instanceof Data) {
			if (((Data) objekt).isArray()) {
				result = new DavDatenFeld(verbindung, (Data) objekt);
			} else {
				result = new DavDatenObjekt(verbindung, (Data) objekt);
			}
		}

		if (result == null) {
			throw new KonfigurationsFehler(UdaFehlerSubtyp.UNBEKANNT, "Kein passendes DAV-Objekt für Typ: "
					+ objekt.getClass().getSimpleName());
		}
		return result;
	}

	/**
	 * das repräsentierte Datenverteilerobjekt.
	 */
	private final Object davObjekt;

	/**
	 * die verwendete Datenverteilerverbindung.
	 */
	private final ClientDavInterface verbindung;

	/**
	 * Konstruktor.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param object
	 *            das Objekt, welches repräsentiert werden soll
	 */
	protected DavDaten(final ClientDavInterface verbindung, final Object object) {
		this.verbindung = verbindung;
		this.davObjekt = object;
	}

	/**
	 * liefert das gekapselte Objekt.
	 *
	 * @return das Objekt
	 */
	public Object getObjekt() {
		return davObjekt;
	}

	/**
	 * liefert die Datenverteiler-Verbindung, über die die Daten für das Objekt
	 * ermittelt wurden.
	 *
	 * @return die Verbindung
	 */
	public ClientDavInterface getVerbindung() {
		return verbindung;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Es wird die Zeichenkettenrepräsentation des Datenverteilerobjekt
	 * ausgegeben.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return davObjekt.toString();
	}
}
