/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.hysterese;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.ListIterator;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;

/**
 * Die Klasse repräsentiert ein Hystereseobjekt innerhalb eines
 * Uda-Skript-Interpreters.<br>
 * Ein Hysteresobjekt dient der Klassifizierung von kontinuierlichen
 * Eingangsgrößen in stufenorientierte Größen.<br>
 * Das Objekt implementiert die Schnittstelle <i>Struktur</i>, um den Zugriff
 * auf Elemente des Objekts über Namen zu realisieren. Folgende Elemente können
 * abgefragt werden:
 * <ul>
 * <li>stufen - liefert die Stufen des Hysteresobjekte</li>
 * </ul>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class Hysterese implements Struktur, Kopierbar {

	/**
	 * die aktuelle Stufe.
	 */
	private HystereseStufe aktuelleStufe;

	/**
	 * das Feld, in dem die Stufendefinitionen des Hystereseobjektes abgelegt
	 * sind.
	 */
	private final HystereseStufenFeld stufen = new HystereseStufenFeld();

	/**
	 * Konstruktor mit Angabe eines Feldes von Hysteresestufen.
	 *
	 * @param stufen
	 *            die Stufen
	 */
	public Hysterese(final HystereseStufe[] stufen) {
		for (final HystereseStufe stufe : stufen) {
			this.stufen.einfuegen(0, stufe);
		}
		if (stufen.length > 0) {
			aktuelleStufe = stufen[0];
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public Kopierbar erzeugeKopie() {
		final Collection<HystereseStufe> stufenFeld = new ArrayList<>();
		for (final Object element : stufen.getElemente()) {
			if (element instanceof HystereseStufe) {
				stufenFeld.add(((HystereseStufe) element).erzeugeKopie());
			}
		}
		final Hysterese result = new Hysterese(stufenFeld.toArray(new HystereseStufe[0]));
		result.setStufe(aktuelleStufe.getName());
		return result;
	}

	/**
	 * Die Funktion liefert das über den übergebenen Namen definierte Element
	 * des Objekts zurück.<br>
	 * Möglich sind folgende Abfragen:
	 * <ul>
	 * <li><b>stufen</b> - liefert die innerhalb des Hystereseobjektes
	 * definierten Stufen</li>
	 * </ul>
	 *
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;

		if ("stufen".equals(name)) {
			result = stufen;
		} else {
			throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT,
					"Der Zugriff auf folgendes Element ist nicht möglich: "
							+ name);
		}
		return result;
	}

	/**
	 * die Funktion liefert die aktuelle Stufe des Hysteresobjekts.
	 *
	 * @return stufe
	 * @see #setStufe(Object)
	 */
	public HystereseStufe getStufe() {
		if (aktuelleStufe == null) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE, "Das Hystereseobjekt hat keine aktuelle Stufe");
		}
		return aktuelleStufe;
	}

	/**
	 * Die Funktion ermittelt die aktuelle Stufe des Hystereseobjekts für den
	 * übergebenen Wert unter Berücksichtigung der letzten Stufe.<br>
	 * Die Abbildung erfolgt nach folgendem Algorithmus:
	 * <ol>
	 * <li>Wenn x im Bereich der letzten Stufe (zwischen Anfangs- und Endwert)
	 * liegt, bleibt die Stufe konstant.</li>
	 * <li>Wenn x größer als der Endwert der letzten Stufe ist, wird die nächst
	 * höhere Stufe, deren Bereich den Wert x beinhaltet, als Ergebnis
	 * zurückgeliefert und diese Stufe bei der Hysterese als letzter Zustand
	 * vermerkt.</li>
	 * <li>Wenn x kleiner als der Anfangswert der letzten Stufe ist, wird die
	 * nächst niedrigere Stufe, deren Bereich den Wert x beinhaltet als Ergebnis
	 * zurüückgeliefert und diese Stufe bei der Hysterese als letzter Zustand
	 * vermerkt.</li>
	 * <li>Wenn bei den Fällen 2. oder 3. keine definierte Stufe gefunden wurde,
	 * deren Bereich den Wert x enthält, wird ein Fehler (ArgumentFehler)
	 * zurückgegeben. Der Wert des letzten Zustands der Hysterese bleibt
	 * bestehen.</li>
	 * </ol>
	 *
	 * @param wert
	 *            der Wert
	 * @return stufe die ermittelte Stufe
	 */
	public Object klassifiziere(final Number wert) {
		final double neuerWert = wert.doubleValue();
		final List<Object> stufenListe = stufen.getElemente();
		final ListIterator<Object> iterator = stufenListe.listIterator(stufenListe.indexOf(aktuelleStufe));
		HystereseStufe neueStufe = null;

		if (aktuelleStufe.unterSchritten(neuerWert)) {
			do {
				if (iterator.hasPrevious()) {
					neueStufe = (HystereseStufe) iterator.previous();
					if (neueStufe.entHaelt(neuerWert)) {
						aktuelleStufe = neueStufe;
						break;
					}
				} else {
					neueStufe = null;
				}
			} while (neueStufe != null);
		} else if (aktuelleStufe.ueberSchritten(neuerWert)) {
			do {
				if (iterator.hasNext()) {
					neueStufe = (HystereseStufe) iterator.next();
					if (neueStufe.entHaelt(neuerWert)) {
						aktuelleStufe = neueStufe;
						break;
					}
				} else {
					neueStufe = null;
				}
			} while (neueStufe != null);
		} else {
			neueStufe = aktuelleStufe;
		}

		if (neueStufe == null) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
					"Der übergebene Wert ist in keiner der definierten Hysteresestufen enthalten");
		}

		return getStufe();
	}

	/**
	 * Die Funktion überschreibt die Daten des mit dem übergebenen Namen
	 * definierten Elements des Objekts mit den üübergebenen.<br>
	 * Für Hysteresobjekte wird diese Funktion nicht unterstützt, d.h. es wird
	 * in jedem Fall ein ObjektZugriffsFehler geworfen. {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
				"Der schreibende Zugriff auf Elemente eines Hystereseobjekts ist nicht möglich.");
	}

	/**
	 * die Funktion setzt die aktuelle Stufe des Hystereseobjekts.
	 *
	 * @param neueStufe
	 *            der Name der Stufe
	 * @see #getStufe()
	 */
	public void setStufe(final Object neueStufe) {
		String name = null;
		if (neueStufe instanceof String) {
			name = (String) neueStufe;
		} else if (neueStufe instanceof HystereseStufe) {
			name = ((HystereseStufe) neueStufe).getName();
		} else {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE);
		}

		for (final Object obj : stufen.getElemente()) {
			final HystereseStufe stufe = (HystereseStufe) obj;
			if (stufe.getName().equalsIgnoreCase(name)) {
				aktuelleStufe = stufe;
				return;
			}
		}
		throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE, "Es ist keine Stufe mit dem angegebenen Namen definiert");
	}

	/**
	 * Die Funktion liefert eine Zeichenkettendarstellung des Hysteresobjekts.
	 * <br>
	 * Die Darstellung erfolgt in der Form
	 * "[(&lt;stufe&gt;),(&lt;stufe&gt;,...)]".<br>
	 * {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer("[");
		for (final Object stufe : stufen.getElemente()) {
			buffer.append('(');
			buffer.append(stufe);
			buffer.append(')');
		}
		buffer.append("-->");
		if (aktuelleStufe == null) {
			buffer.append("keine aktuelle Stufe");
		} else {
			buffer.append(aktuelleStufe.getName());
		}
		buffer.append(']');
		return buffer.toString();
	}

}
