/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.struktur;

import java.util.Arrays;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.OperandenListe;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Testklasse zum Test des Handlers für Datenstrukturen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DatenStrukturHandlerTest {

	/**
	 * der verwendete Handler.
	 */
	private static final DatenStrukturHandler HANDLER = new DatenStrukturHandler();

	/**
	 * der Operator, der jeweils getestet werden soll.
	 */
	private Operator operator;

	/**
	 * testet die Verfügbarkeit aller erwarteten Operatoren.
	 */
	@Test
	public void testGetHandledOperators() {
		Assert.assertTrue(DatenStrukturHandlerTest.HANDLER.getHandledOperators().length > 0);
		Assert.assertTrue(Arrays.asList(DatenStrukturHandlerTest.HANDLER.getHandledOperators())
				.contains(DatenStrukturOperatoren.DATENSATZ));
		Assert.assertTrue(Arrays.asList(DatenStrukturHandlerTest.HANDLER.getHandledOperators())
				.contains(DatenStrukturOperatoren.DATENSTRUKTUR));
		Assert.assertTrue(Arrays.asList(DatenStrukturHandlerTest.HANDLER.getHandledOperators())
				.contains(DatenStrukturOperatoren.KONSTRUKTOR));
	}

	/**
	 * testet die Gültigkeit des Konstruktors für verschiedene Operandenlisten.
	 */
	@Test
	public void testIsHandlerKonstruktor() {
		operator = DatenStrukturOperatoren.KONSTRUKTOR;
		Assert.assertFalse(DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get()).isValid());
		Assert.assertTrue(DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get("x"))
				.isValid());
		Assert.assertTrue(DatenStrukturHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get("y", "z", "a", "b", "c", "d")).isValid());
	}

	/**
	 * testet die Gültigkeit des Operators "datensatz" für verschiedene
	 * Operandenlisten.
	 */
	@Test
	public void testIsHandlerDatensatz() {
		final DatenStrukturDefinition definition = new DatenStrukturDefinition(new String[] { "x", "y", "z" });
		operator = DatenStrukturOperatoren.DATENSATZ;

		Assert.assertFalse(DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get()).isValid());
		Assert.assertFalse(
				DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get("Heinz")).isValid());
		Assert.assertTrue(
				DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get(definition)).isValid());
		Assert.assertTrue(DatenStrukturHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(definition, "Heinz")).isValid());
		Assert.assertFalse(DatenStrukturHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(definition, "Heinz", "Heinz", "Heinz",
						"Heinz", "Heinz", "Heinz", "Heinz"))
				.isValid());
	}

	/**
	 * testet die Gültigkeit des Operators "datenstruktur" für verschiedene
	 * Operandenlisten.
	 */
	@Test
	public void testIsHandlerDatenStruktur() {
		operator = DatenStrukturOperatoren.DATENSTRUKTUR;
		final DatenStrukturDefinition definition = new DatenStrukturDefinition(new String[] { "x", "y", "z" });
		final DatenStruktur struktur = new DatenStruktur(definition, new Object[3]);

		Assert.assertFalse(DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get()).isValid());
		Assert.assertFalse(
				DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get("Heinz")).isValid());
		Assert.assertTrue(
				DatenStrukturHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get(struktur)).isValid());
		Assert.assertFalse(DatenStrukturHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(struktur, "Heinz")).isValid());
	}

	/**
	 * testet die Ausführung des Konstruktors "Datenstruktur" für verschiedene
	 * Operandenlisten.
	 */
	@Test
	public void testPerformKonstruktor() {
		operator = DatenStrukturOperatoren.KONSTRUKTOR;

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get());
			Assert.fail("Konstruktoraufruf mit ungültigen Operanden");
		} catch (final ArgumentFehler e) {
			// Keine Konstruktion möglich
		}

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get(1));
			Assert.fail("Konstruktoraufruf mit ungültigen Operanden");
		} catch (final ArgumentFehler e) {
			// Keine Konstruktion möglich
		}

		DatenStrukturDefinition definition = (DatenStrukturDefinition) DatenStrukturHandlerTest.HANDLER
				.perform(operator, OperandenListe.get("x"));
		Assert.assertEquals("{x}", definition.toString());

		definition = (DatenStrukturDefinition) DatenStrukturHandlerTest.HANDLER.perform(operator,
				OperandenListe.get("x", "y", "z", "a", "b", "c", "d"));
		Assert.assertEquals("{x,y,z,a,b,c,d}", definition.toString());
	}

	/**
	 * testet die Ausführung des Operators "datensatz" für verschiedene
	 * Operandenlisten.
	 */
	@Test
	public void testPerformDatensatz() {
		final DatenStrukturDefinition definition = new DatenStrukturDefinition(new String[] { "x", "y", "z" });
		operator = DatenStrukturOperatoren.DATENSATZ;

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get());
			Assert.fail("Funktionsaufruf mit ungültigen Operanden");
		} catch (final ArgumentFehler e) {
			// Funktion nicht ausgeführt
		}

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get("Heinz"));
			Assert.fail("Funktionsaufruf mit ungültigen Operanden");
		} catch (final ArgumentFehler e) {
			// Funktion nicht ausgeführt
		}

		Assert.assertEquals("{x=undefiniert,y=undefiniert,z=undefiniert}", DatenStrukturHandlerTest.HANDLER
				.perform(operator, OperandenListe.get(definition)).toString());

		Assert.assertEquals("{x=Heinz,y=undefiniert,z=undefiniert}",
				DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get(definition, "Heinz")).toString());

		Assert.assertEquals("{x=Heinz,y=Heinz,z=Heinz}",
				DatenStrukturHandlerTest.HANDLER
				.perform(operator, OperandenListe.get(definition, "Heinz", "Heinz", "Heinz"))
				.toString());

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get(definition, "Heinz",
					"Heinz", "Heinz", "Heinz"));
			Assert.fail("Funktionsaufruf mit zu vielen Initalwerten");
		} catch (final ArgumentFehler e) {
			// Funktion nicht ausgeführt
		}
	}

	/**
	 * testet die Ausführung des Operators "datenstruktur" für verschiedene
	 * Operandenlisten.
	 */
	@Test
	public void testPerformDatenStruktur() {
		operator = DatenStrukturOperatoren.DATENSTRUKTUR;
		final DatenStrukturDefinition definition = new DatenStrukturDefinition(new String[] { "x", "y", "z" });
		final DatenStruktur struktur = new DatenStruktur(definition, new Object[3]);

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get());
			Assert.fail("Funktionsaufruf mit ungültigen Operanden");
		} catch (final ArgumentFehler e) {
			// Funktion nicht ausgeführt
		}

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get("Heinz"));
			Assert.fail("Funktionsaufruf mit ungültigen Operanden");
		} catch (final ArgumentFehler e) {
			// Funktion nicht ausgeführt
		}

		Assert.assertEquals(definition,
				DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get(struktur)));

		try {
			DatenStrukturHandlerTest.HANDLER.perform(operator, OperandenListe.get(struktur, "Heinz"));
			Assert.fail("Funktionsaufruf mit zu vielen Operanden");
		} catch (final ArgumentFehler e) {
			// Funktion nicht ausgeführt
		}
	}
}
