/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.trend;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitDauer;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Die Repräsentation eines Vergangenheits-"Operators".<br>
 * Ein Trendobjekt kann Wertepaaren (Zahl, Zeitstempel) gefüllt werden, wobei
 * die Zeitstempel chronologisch korrekt sein müssen. Das Objekt bietet
 * Operationen zur Bestimmung der Monotonie der übergebenen Werte.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TrendObjekt implements Kopierbar {

	/**
	 * die Kapazit&auml; des Trendobjekts, d.h. die Anzahl der darin
	 * speicherbaren Eintr&auml;ge.
	 */
	private final int kapazitaet;

	/**
	 * das Objekt, auf das sich der trend bezieht.<br>
	 * Das Objekt wird durch den Nutzer angegeben, kann jeden beliebigen Typ
	 * haben und wird innerhalb des Objektes selbst nicht weiter verwendet.
	 */
	private Object bezug = NichtWert.NICHTWERT;

	/**
	 * die Liste der Trendeinträge.
	 */
	private final List<TrendEintrag> werte = new ArrayList<>();

	/**
	 * Konstruktor mit Angabe von Bezug und Kapazit&auml;t.
	 *
	 * @param bezug
	 *            der Bezug
	 * @param kapazitaet
	 *            die Kapazit&auml;t, der Wert 0 steht für unbegrenzt.
	 */
	public TrendObjekt(final Object bezug, final int kapazitaet) {
		if (bezug != null) {
			this.bezug = bezug;
		}
		this.kapazitaet = kapazitaet;
	}

	/**
	 * fügt dem Trendobjekt einen Wert hinzu.<br>
	 * Es können nur zeitrichtig fortlaufende Werte &uuml;bergeben werden. Wird
	 * kein Zeitstempel &uuml;bergeben, wird die aktuelle Zeit verwendet.
	 *
	 * @param wert
	 *            der Wert
	 * @param zeit
	 *            der Zeitstempel
	 */
	public void addWert(final Number wert, final Zeitstempel zeit) {
		Zeitstempel zeitWert = zeit;
		if (zeit == null) {
			zeitWert = new Zeitstempel(null);
		}
		long lastZeitWert = 0L;
		if (werte.size() > 0) {
			lastZeitWert = werte.get(0).getZeit();
		}
		if (lastZeitWert > zeitWert.getMilliSekunden()) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
					"Der Zeitstempel liegt vor dem des zuletzt hinzugefügten Wert");
		}

		werte.add(0, new TrendEintrag(zeitWert, wert));

		if (kapazitaet > 0) {
			if (werte.size() > kapazitaet) {
				werte.remove(werte.size() - 1);
			}
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public TrendObjekt erzeugeKopie() {
		final TrendObjekt result = new TrendObjekt(bezug, kapazitaet);
		for (final TrendEintrag eintrag : werte) {
			result.werte.add(eintrag.erzeugeKopie());
		}
		return result;
	}

	/**
	 * liefert die Anzahl der Eintr&auml;ge innerhalb des Trendobjekts.
	 *
	 * @return die Anzahl
	 */
	public Number getAnzahl() {
		return werte.size();
	}

	/**
	 * liefert das Objekt, auf das sich das Trendobjekt bezieht.
	 *
	 * @return den Bezug
	 */
	public Object getBezug() {
		return bezug;
	}

	/**
	 * liefert die Anzahl der streng monoton fallenden Werte ausgehend vom
	 * zuletzt gespeicherten.
	 *
	 * @return die Anzahl
	 */
	public Long getMaxFallend() {
		long count = 0;
		TrendEintrag last = null;
		for (final TrendEintrag eintrag : werte) {
			if (last != null) {
				if (!eintrag.fallend(last).getBoolWert()) {
					break;
				}
			}
			last = eintrag;
			count++;
		}
		return count;
	}

	/**
	 * liefert die Anzahl der konstanten Werte ausgehend vom zuletzt
	 * gespeicherten.
	 *
	 * @return die Anzahl
	 */
	public Long getMaxKonstant() {
		long count = 0;
		TrendEintrag last = null;
		for (final TrendEintrag eintrag : werte) {
			if (last != null) {
				if (!eintrag.konstant(last).getBoolWert()) {
					break;
				}
			}
			last = eintrag;
			count++;
		}
		return count;
	}

	/**
	 * liefert die Anzahl der streng monoton steigenden Werte ausgehend vom
	 * zuletzt gespeicherten.
	 *
	 * @return die Anzahl
	 */
	public Long getMaxSteigend() {
		long count = 0;
		TrendEintrag last = null;
		for (final TrendEintrag eintrag : werte) {
			if (last != null) {
				if (!eintrag.steigend(last).getBoolWert()) {
					break;
				}
			}
			last = eintrag;
			count++;
		}
		return count;
	}

	/**
	 * liefert den zuletzt gespeicherten Wert des Trendobjekts.
	 *
	 * @return den Wert
	 */
	public Object getWert() {
		if (werte.isEmpty()) {
			return NichtWert.NICHTWERT;
		}
		return werte.get(0).getWert();
	}

	/**
	 * liefert eine Teilliste aus den innerhalb des Trendobjekts gespeicherten
	 * Einträgen.<br>
	 * F&uuml;r die Auswahl wird ein optionaler Anfangszeitstempel (Standard:
	 * aktuelle Zeit) und eine Anzahl bzw. eine Zeitdauer verwendet.
	 *
	 * @param parameter
	 *            Begrenzung für die Auswahl (Zeitdauer oder Anzahl)
	 * @param zeitStempel
	 *            der Zeitstempel des Anfangspunktes für die Auswahl
	 * @return die Liste mit den g&uuml;ltigen Eintr&auml;gen
	 */
	private List<TrendEintrag> getWerteListe(final Object parameter, final Zeitstempel zeitStempel) {
		final List<TrendEintrag> result = new ArrayList<>();
		if (parameter == null) {
			result.addAll(werte);
		} else {
			long startZeit = new Date().getTime();
			if (zeitStempel != null) {
				startZeit = zeitStempel.getMilliSekunden();
			}

			if (werte.size() <= 0) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE, "Trendobjekt enthält keine Daten");
			}

			final long ersteZeit = werte.get(werte.size() - 1).getZeit();
			if (startZeit < ersteZeit) {
				throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
						"Abfragezeit liegt vor dem ältesten Eintrag innerhalb des Trendobjekts");
			}

			long endZeit = 0L;
			if (parameter instanceof ZeitDauer) {
				endZeit = startZeit - ((ZeitDauer) parameter).getMilliSekunden();
			}

			long count = Long.MAX_VALUE;
			if (parameter instanceof Number) {
				count = ((Number) parameter).longValue();
			}

			final Iterator<TrendEintrag> iterator = werte.iterator();
			while (iterator.hasNext()) {
				final TrendEintrag eintrag = iterator.next();
				if (eintrag.getZeit() <= startZeit) {
					if (result.size() >= count) {
						break;
					}
					if (eintrag.getZeit() <= endZeit) {
						break;
					}
					result.add(eintrag);
				}
			}

			if (count != Long.MAX_VALUE) {
				if (result.size() < count) {
					result.add(null);
				}
			} else {
				if ((result.size() > 0) && (endZeit < ersteZeit)) {
					result.add(null);
				}
			}
		}

		return result;
	}

	/**
	 * liefert den Zeitstempel des zuletzt gespeicherten Wertes.
	 *
	 * @return den Zeitstempel
	 */
	public Object getZeit() {
		if (werte.isEmpty()) {
			return NichtWert.NICHTWERT;
		}
		return new Zeitstempel(werte.get(0).getZeit());
	}

	/**
	 * ermittelt, ob die Werte des Trendobjektes monoton fallend sind.<br>
	 * Es werden alle Werte ber&uuml;chsichtigt, die &uuml;ber die
	 * &uuml;bergebenen Parameter aud der Menge aller Eintr&auml;ge des
	 * Trendobjekts ausgew&auml;hlt werden.
	 *
	 * @param parameter
	 *            der Auswahlparameter (Anzahl oder Zeitdauer)
	 * @param zeitStempel
	 *            der Startpunkt für die Suche (Standard: aktueller Zeitpunkt)
	 * @return wahr, wenn die ausgewählten Werte monoton fallend sind
	 */
	public LogischerWert isFallend(final Object parameter, final Zeitstempel zeitStempel) {
		boolean result = false;
		TrendEintrag last = null;
		final List<TrendEintrag> liste = getWerteListe(parameter, zeitStempel);
		if (!liste.isEmpty()) {
			result = true;
			for (final TrendEintrag eintrag : getWerteListe(parameter, zeitStempel)) {
				if (eintrag == null) {
					result = false;
					break;
				}
				if (last != null) {
					if (eintrag.steigend(last).getBoolWert()) {
						result = false;
						break;
					}
				}
				last = eintrag;
			}
		}

		return LogischerWert.valueOf(result);
	}

	/**
	 * ermittelt, ob die Werte des Trendobjektes konstant sind.<br>
	 * Es werden alle Werte ber&uuml;chsichtigt, die &uuml;ber die
	 * &uuml;bergebenen Parameter aud der Menge aller Eintr&auml;ge des
	 * Trendobjekts ausgew&auml;hlt werden.
	 *
	 * @param parameter
	 *            der Auswahlparameter (Anzahl oder Zeitdauer)
	 * @param zeitStempel
	 *            der Startpunkt für die Suche (Standard: aktueller Zeitpunkt)
	 * @return wahr, wenn die ausgewählten Werte konstant sind
	 */
	public LogischerWert isKonstant(final Object parameter, final Zeitstempel zeitStempel) {
		boolean result = false;
		TrendEintrag last = null;
		final List<TrendEintrag> liste = getWerteListe(parameter, zeitStempel);
		if (!liste.isEmpty()) {
			result = true;
			for (final TrendEintrag eintrag : liste) {
				if (eintrag == null) {
					result = false;
					break;
				}
				if (last != null) {
					if (!eintrag.konstant(last).getBoolWert()) {
						result = false;
						break;
					}
				}
				last = eintrag;
			}
		}
		return LogischerWert.valueOf(result);
	}

	/**
	 * ermittelt, ob die Werte des Trendobjektes monoton steigend sind.<br>
	 * Es werden alle Werte ber&uuml;chsichtigt, die &uuml;ber die
	 * &uuml;bergebenen Parameter aud der Menge aller Eintr&auml;ge des
	 * Trendobjekts ausgew&auml;hlt werden.
	 *
	 * @param parameter
	 *            der Auswahlparameter (Anzahl oder Zeitdauer)
	 * @param zeitStempel
	 *            der Startpunkt für die Suche (Standard: aktueller Zeitpunkt)
	 * @return wahr, wenn die ausgewählten Werte monoton steigend sind
	 */
	public LogischerWert isSteigend(final Object parameter, final Zeitstempel zeitStempel) {
		boolean result;
		result = false;
		TrendEintrag last = null;
		final List<TrendEintrag> liste = getWerteListe(parameter, zeitStempel);
		if (!liste.isEmpty()) {
			result = true;
			for (final TrendEintrag eintrag : liste) {
				if (eintrag == null) {
					result = false;
					break;
				}
				if (last != null) {
					if (eintrag.fallend(last).getBoolWert()) {
						result = false;
						break;
					}
				}
				last = eintrag;
			}
		}
		return LogischerWert.valueOf(result);
	}
}
