/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.skript.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.bsvrz.ibv.uda.client.LokalesSkript;
import de.bsvrz.ibv.uda.client.SkriptLaufDaten;
import de.bsvrz.ibv.uda.interpreter.InterpreterListener;
import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.interpreter.UdaInterpreterException;
import de.bsvrz.ibv.uda.verwaltung.protokoll.DateiProtokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.KonsoleProtokoll;

/**
 * Führt eine vorgegebene Anzahl von Uda-Skripten aus und vergleicht die
 * erhaltenen Asugaben mit den erwarteten.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SkriptsIT implements InterpreterListener {

	/**
	 * der Skriptlauf, der im jeweils aktiven Interpreter ausgeführt wird.
	 */
	private static final SkriptsIT INSTANCE = new SkriptsIT();

	/**
	 * der Skriptlauf, der im jeweils aktiven Interpreter ausgeführt wird.
	 */
	private final SkriptLaufDaten lauf = new SkriptLaufDaten(null, null, null);

	/**
	 * der Pfad, in dem die Skript- und Ergebnisdateien abgelegt sind.
	 */
	private String path = System.getenv("UDA_JUNIT_SKRIPTPATH");

	/**
	 * Mutex zum synchronisieren der Beendigung eines Interpreters mit dem Test
	 * der Ergebnisdatei.
	 */
	private final Object mutex = new Object();

	/**
	 * markiert, ob die Ausgaben eines Skripts auch auf der Konsole ausgegeben
	 * werden sollen.
	 */
	private boolean konsolenAusgabe;

	/**
	 * führt das Skript "Ausdruecke.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void ausdrucke() throws UdaInterpreterException {
		skriptAusfuehren(path, "Ausdruecke.uda", "Ausdruecke.result");
	}

	/**
	 * führt das Skript "Basisfunktionen.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void basisFunktionen() throws UdaInterpreterException {
		skriptAusfuehren(path, "Basisfunktionen.uda", "Basisfunktionen.result");
	}

	/**
	 * {@inheritDoc}.<br>
	 * Triggert den aktuell ausgeführten Interpreter, wenn die Ausführung des
	 * Skriptes beendet wurde.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.InterpreterListener#beendet(de.bsvrz.ibv.uda.interpreter.UdaInterpreter)
	 */
	@Override
	public void beendet(final UdaInterpreter interpreter) {
		synchronized (mutex) {
			mutex.notifyAll();
		}
	}

	/**
	 * führt das Skript "Datenstrukturen.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void datenStrukturen() throws UdaInterpreterException {
		skriptAusfuehren(path, "Datenstrukturen.uda", "Datenstrukturen.result");
	}

	/**
	 * führt das Skript "Fehler.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void fehler() throws UdaInterpreterException {
		skriptAusfuehren(path, "Fehler.uda", "Fehler.result");
	}

	/**
	 * führt das Skript "FelderUndListen.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void felderUndListen() throws UdaInterpreterException {
		skriptAusfuehren(path, "FelderUndListen.uda", "FelderUndListen.result");
	}

	/**
	 * führt das Skript "Fuzzy.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void fuzzy() throws UdaInterpreterException {
		skriptAusfuehren(path, "Fuzzy.uda", "Fuzzy.result");
	}

	/**
	 * führt das Skript "Gueltigkeitsbereich.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void gueltigkeit() throws UdaInterpreterException {
		skriptAusfuehren(path, "Gueltigkeitsbereich.uda", "Gueltigkeitsbereich.result");
	}

	/**
	 * führt das Skript "Hysterese.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void hysterese() throws UdaInterpreterException {
		skriptAusfuehren(path, "Hysterese.uda", "Hysterese.result");
	}

	/**
	 * führt das Skript "Iteration.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void iteration() throws UdaInterpreterException {
		skriptAusfuehren(path, "Iteration.uda", "Iteration.result");
	}

	/**
	 * führt das Skript "Java.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void java() throws UdaInterpreterException {
		skriptAusfuehren(path, "Java.uda", "Java.result");
	}

	/**
	 * führt das Skript "KonfigurationUndMetadaten.uda" aus und prüft die
	 * Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void konfiguration() throws UdaInterpreterException {
		skriptAusfuehren(path, "KonfigurationUndMetadaten.uda", "KonfigurationUndMetadaten.result");
	}

	/**
	 * führt das Skript "Konstante.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void konstante() throws UdaInterpreterException {
		skriptAusfuehren(path, "Konstante.uda", "Konstante.result");
	}

	/**
	 * vergleicht zwei Dateien zeilenweise und löst eine Assertion aus, wenn
	 * eine Zeile keine Übereinstimmung erzielt.
	 *
	 * @param ergebnis
	 *            der Name der Ergebnisdatei
	 * @param erwartet
	 *            der Name der Datei, die die erwarteteten Ergebnisse enthält
	 */
	private void pruefeErgebnis(final String ergebnis, final String erwartet) {
		try {
			final BufferedReader ergebnisIo = new BufferedReader(new FileReader(ergebnis));
			final BufferedReader erwartetIo = new BufferedReader(
					new InputStreamReader(SkriptsIT.INSTANCE.getClass().getResourceAsStream(erwartet)));

			String erwartetZeile = null;
			String ergebnisZeile = null;
			int zeile = 1;
			do {
				erwartetZeile = erwartetIo.readLine();
				if (erwartetZeile != null) {
					erwartetZeile = erwartetZeile.trim();
					ergebnisZeile = ergebnisIo.readLine();
					if (ergebnisZeile != null) {
						ergebnisZeile = ergebnisZeile.trim();
					}
					if (!erwartetZeile.equals(ergebnisZeile)) {
						Assert.fail(new File(ergebnis).getName() + " (" + zeile
								+ "): Erwartet: " + erwartetZeile
								+ ", Erhalten: " + ergebnisZeile);
						break;
					}
				}
				zeile++;
			} while (erwartetZeile != null);
		} catch (final FileNotFoundException e) {
			Assert.fail(e.getMessage());
		} catch (final IOException e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * führt das Skript "Quantoren.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void quantoren() throws UdaInterpreterException {
		skriptAusfuehren(path, "Quantoren.uda", "Quantoren.result");
	}

	/**
	 * führt das Skript "SchluesselMengen.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void schluesselMengen() throws UdaInterpreterException {
		skriptAusfuehren(path, "SchluesselMengen.uda", "SchluesselMengen.result");
	}

	/**
	 * initialisiert die global verwendeten Testobjekte.
	 */
	@Before
	public void setup() {

		lauf.setBenutzer("Tester");
		lauf.setPasswd("geheim");
		lauf.setParameter("-p1", "Parameter 1");
		lauf.setParameter("-p2", "Parameter 2");
		lauf.setParameter("-p3", "Parameter 3");

		if (System.getenv("KONSOLENAUSGABE") != null) {
			konsolenAusgabe = true;
		}

		if (path == null) {
			// path = "./junit/de/bsvrz/ibv/uda/interpreter/skript/test";
			//
			path = "de/bsvrz/ibv/uda/interpreter/skript/test";
			konsolenAusgabe = true;
		}
	}

	/**
	 * führt ein Skript aus und vergleicht die Ergebnisse mit den vorgegebenen.
	 *
	 * @param skriptPath
	 *            Pfad, in dem die Skripte zu finden sind
	 * @param dateiName
	 *            der Name der Datei mit dem Skript-Quelltext
	 * @param ergebnisDatei
	 *            der Name der Datei, die die erwarteten Ergebnisse enthält
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	private void skriptAusfuehren(final String skriptPath, final String dateiName, final String ergebnisDatei)
			throws UdaInterpreterException {
		try {
			System.err.println("Initialisiere SkriptLauf für Datei: "
					+ dateiName);
			lauf.setSkript(new LokalesSkript(dateiName, SkriptsIT.INSTANCE.getClass().getResourceAsStream(dateiName)));
			lauf.clearProtokolle();
			if (konsolenAusgabe) {
				lauf.addProtokoll(new KonsoleProtokoll());
			}
			final DateiProtokoll dateiProtokoll = new DateiProtokoll();
			final File ausgabeFile;
			ausgabeFile = File.createTempFile("udatest", ".check");
			ausgabeFile.deleteOnExit();
			dateiProtokoll.setName(ausgabeFile.getAbsolutePath());
			lauf.addProtokoll(dateiProtokoll);
			final UdaInterpreter interpreter = new UdaInterpreter(null, lauf);
			interpreter.addListener(this);
			interpreter.start();

			synchronized (mutex) {
				mutex.wait();
			}

			// pruefeErgebnis(ausgabeFile.getAbsolutePath(), path +
			// File.separator
			// + ergebnisDatei);
			pruefeErgebnis(ausgabeFile.getAbsolutePath(), ergebnisDatei);
		} catch (final IOException e1) {
			Assert.fail(e1.getMessage());
		} catch (final InterruptedException e) {
			e.printStackTrace();
		}

	}

	/**
	 * führt das Skript "SpezielleObjekte.uda" aus und prüft die Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void spezielleObjekte() throws UdaInterpreterException {
		skriptAusfuehren(path, "SpezielleObjekte.uda", "SpezielleObjekte.result");
	}

	/**
	 * führt das Skript "VergangenheitsOperatoren.uda" aus und prüft die
	 * Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void trend() throws UdaInterpreterException {
		skriptAusfuehren(path, "VergangenheitsOperatoren.uda", "VergangenheitsOperatoren.result");
	}

	/**
	 * führt das Skript "TypenUndKonvertierung.uda" aus und prüft die
	 * Ergebnisse.
	 *
	 * @throws UdaInterpreterException
	 *             die Skriptausführung war nicht erfolgreich
	 */
	@Test
	public void typen() throws UdaInterpreterException {
		skriptAusfuehren(path, "TypenUndKonvertierung.uda", "TypenUndKonvertierung.result");
	}
}
