/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.uda;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.ibv.uda.client.LokalesSkript;
import de.bsvrz.ibv.uda.client.SkriptLaufDaten;
import de.bsvrz.ibv.uda.interpreter.InterpreterListener;
import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.interpreter.UdaInterpreterException;
import de.bsvrz.ibv.uda.uda.data.SkriptLauf;
import de.bsvrz.ibv.uda.verwaltung.protokoll.DateiProtokoll;
import de.bsvrz.ibv.uda.verwaltung.protokoll.KonsoleProtokoll;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList.Argument;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Implementierung der Datenverteiler-Standardapplikation zur Ausführung eines
 * lokalen Skriptlaufs.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaSkriptAusfuehren implements StandardApplication, InterpreterListener {

	/**
	 * Standard-Suffix für Protokolldateien.
	 */
	private static final String PROTOKOLL_FILE_SUFFIX = ".prot";

	/**
	 * Standard-Suffix für UDA-Skript-Quelltext-Dateien.
	 */
	private static final String UDA_FILE_SUFFIX = ".uda";

	/**
	 * Parametername zur Übergabe des Namens der Skriptdatei.
	 */
	private static final String PARAMETER_SKRIPTNAME = "-skriptName";

	/**
	 * Parametername zur optionaln Übergabe des Namens der Protokolldatei.
	 */
	private static final String PARAMETER_PROTOKOLLNAME = "-protokollName";

	/**
	 * Parametername zur optionalen Aktivierung des Konsolenprotokolls.
	 */
	private static final String PARAMETER_KONSOLENAUSGABE = "-konsolenAusgabe";

	/**
	 * der Name der Datei mit dem Skript-Quelltext.
	 */
	private String skriptName = "";

	/**
	 * die Liste der an das Skript übergebenen Parameter.
	 */
	private final Map<String, String> parameterListe = new HashMap<>();

	/**
	 * der Name der Datei für das Protokoll des Skriptlaufs.
	 */
	private String protokollName;

	/**
	 * die verwendete Datenverteilerverbindung.
	 */
	private ClientDavInterface verbindung;

	/**
	 * Marker, ob die Konsolenausgabe eingeschaltet werden soll oder nicht.
	 */
	private boolean konsolenAusgabe;

	/** Logger für Debugausgaben. */
	private Debug logger;

	/** {@inheritDoc} */
	@Override
	public void beendet(final UdaInterpreter interpreter) {
		verbindung.disconnect(false, "Skriptausführung: " + skriptName
				+ " beendet");
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
	 */
	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {

		verbindung = connection;

		final SkriptLauf lauf = new SkriptLaufDaten(null, "LokalerSkriptlauf: "
				+ skriptName, new LokalesSkript(skriptName));
		for (final String key : parameterListe.keySet()) {
			lauf.setParameter(key, parameterListe.get(key));
		}

		if (protokollName != null) {
			final DateiProtokoll protokoll = new DateiProtokoll();
			protokoll.setName(protokollName);
			lauf.addProtokoll(protokoll);
		}

		if (konsolenAusgabe) {
			lauf.addProtokoll(new KonsoleProtokoll());
		}

		lauf.setBenutzer(connection.getClientDavParameters().getUserName());
		lauf.setPasswd(connection.getClientDavParameters().getUserPassword());
		try {
			final UdaInterpreter interpreter = new UdaInterpreter(connection, lauf);
			interpreter.addListener(this);
			interpreter.start();

		} catch (final UdaInterpreterException e) {
			e.printStackTrace();
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#parseArguments(de.bsvrz.sys.funclib.commandLineArgs.ArgumentList)
	 */
	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {

		logger = Debug.getLogger();

		while (argumentList.hasUnusedArguments()) {
			skriptName = argumentList.fetchArgument(UdaSkriptAusfuehren.PARAMETER_SKRIPTNAME + "=")
					.asNonEmptyString();

			if (argumentList.hasArgument(UdaSkriptAusfuehren.PARAMETER_PROTOKOLLNAME)) {
				final String name = argumentList.fetchArgument(UdaSkriptAusfuehren.PARAMETER_PROTOKOLLNAME).asString();
				if ((name != null) && (name.length() > 0)) {
					protokollName = name;
				}
			}

			if (argumentList.hasArgument(UdaSkriptAusfuehren.PARAMETER_KONSOLENAUSGABE)) {
				konsolenAusgabe = argumentList.fetchArgument(UdaSkriptAusfuehren.PARAMETER_KONSOLENAUSGABE)
						.booleanValue();
			}

			while (argumentList.hasUnusedArguments()) {
				final Argument arg = argumentList.fetchNextArgument();
				parameterListe.put(arg.getName(), arg.getValue());
			}
		}

		if (!konsolenAusgabe) {
			if ((protokollName == null) || (protokollName.length() <= 0)) {
				if (skriptName.endsWith(UdaSkriptAusfuehren.UDA_FILE_SUFFIX)) {
					protokollName = skriptName.substring(0,
							skriptName.length() - UdaSkriptAusfuehren.UDA_FILE_SUFFIX.length())
							+ UdaSkriptAusfuehren.PROTOKOLL_FILE_SUFFIX;
				} else {
					protokollName = skriptName + UdaSkriptAusfuehren.PROTOKOLL_FILE_SUFFIX;
				}
			}
		}

		if (skriptName == null) {
			logger.error("Manueller Skriptlauf kann nicht ausgeführt werden, da kein Skript übergeben wurde");
		}
	}
}
