/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.verwaltung.protokoll;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.uda.data.SkriptLauf;
import de.bsvrz.ibv.uda.verwaltung.ServerSkriptLauf;

/**
 * beschreibt die Protokollierung der Ausgaben eines Uda-Skripts über eine
 * Attributgruppe des Skriptlaufes.<br>
 * Der Skriptlauf einthält die Attributgruppe atg.udaSkriptLaufLogger, die aus
 * einer variablen Liste von Textzeilen besteht, in der die Ausgabemeldungen des
 * Skripts bzw. Skriptlaufs eingetragen werden können. Die eigentliche Ausgabe
 * erfolgt über die Skriptlaufklasse selbst.<br>
 * Die Anzahl der innerhalb der Attributgruppe übertragen Ausgabezeilen wir
 * durch die innerhalb des Protokolleintrags definierte Anzahl definiert.<br>
 * Ein Protokoll dieses Typs kann nur von einem Skriptlaufm, der auf dem Server
 * "Umfassende Datenanalyse" ausgeführt wird erzeugt werden.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DavProtokoll extends Protokoll {

	/**
	 * die Versions-ID für die Serialisierung.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * der Skriptlauf, dessen Ausgaben protokolliert werden sollen.
	 */
	private transient ServerSkriptLauf serverLauf;

	/**
	 * die Liste der Meldungen, die gepuffert werden sollen.<br>
	 * Die Größe der Liste wird durch durch die Angabe der Anzahl bei der
	 * Definition des Protokoll begrenzt.
	 */
	private transient List<String> meldungsListe;

	/**
	 * Standardkonstruktor.
	 */
	public DavProtokoll() {
		super(ProtokollTyp.DAV);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die empfangenen Ausgabedaten werden in den internen Meldungspuffer
	 * übertragen, dieser wird bei Bedarf auf die maximale Länge verkürzt. Die
	 * Ausgabe der Meldungsliste erfolgt über das Skriptlaufobjekt.<br>
	 * Die maximale Länge der Liste beträgt mindestens 1, d.h. wenn eine
	 * maximale Länge &lt;= 0 angegeben wurde, wird diese automatisch auf 1
	 * korrigiert.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#ausgeben(de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollEintrag)
	 */
	@Override
	public void ausgeben(final ProtokollEintrag daten) {
		if (serverLauf != null) {
			if (getAnzahl() <= 0) {
				setAnzahl(1);
			}

			while (meldungsListe.size() >= getAnzahl()) {
				meldungsListe.remove(meldungsListe.size() - 1);
			}

			meldungsListe.add(0, daten.getText());
			serverLauf.ausgeben(meldungsListe);
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion leert den internen Meldungspuffer des Objekts bzw. legt ihn
	 * bei Bedarf an.<br>
	 * Desweiteren wird der Skriptlauf, dessen Ausgaben protokolliert werden und
	 * über den die Ausgabe der Daten erfolgen soll zugeordnet.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#initialisiere(de.bsvrz.ibv.uda.uda.data.SkriptLauf)
	 */
	@Override
	public void initialisiere(final SkriptLauf lauf) {
		if (meldungsListe == null) {
			meldungsListe = new ArrayList<>();
		}
		meldungsListe.clear();
		if (lauf instanceof ServerSkriptLauf) {
			this.serverLauf = (ServerSkriptLauf) lauf;
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion leert die interne Meldungsliste und hebt die Zuordnung zum
	 * Skriptlauf auf.
	 *
	 * @see de.bsvrz.ibv.uda.verwaltung.protokoll.Protokoll#abschliessen()
	 */
	@Override
	public void abschliessen() {
		if (meldungsListe != null) {
			meldungsListe.clear();
		}
		serverLauf = null;
	}
}
